from django.test import TestCase

from ocyan.plugin.wagtail_oscar_integration.blocks import ProductChooserBlock


class ProductChooserBlockTestCase(TestCase):
    def test_value_from_form_empty(self):
        block = ProductChooserBlock(required=False)
        self.assertEqual(block.clean(""), None)
        self.assertEqual(block.value_from_form(None), None)



import sys

from wagtail.utils.deprecation import MovedDefinitionHandler

MOVED_DEFINITIONS = {
    "get_wagtailsite": "ocyan.plugin.wagtail_utils.utils",
    "get_wagtailsettings": "ocyan.plugin.wagtail_utils.utils",
    "get_offline_request": "ocyan.plugin.wagtail_utils.utils",
    "get_default_site": "ocyan.plugin.wagtail_utils.utils",
    "StaticSrcFinder": "ocyan.plugin.wagtail_utils.utils",
}

sys.modules[__name__] = MovedDefinitionHandler(
    sys.modules[__name__], MOVED_DEFINITIONS, DeprecationWarning
)



from wagtail import blocks

from oscar.core.loading import get_model

from .blocks import RoadrunnerCategoryChooserBlock

Product = get_model("catalogue", "Product")


# Because we moved oscar related blocks to this plugin,
# we had to maintain backwards compatibility, eg;
# not suddenly losing blocks from a page. This only works if
# the class name of the block is the same. But in blocks.py
# there already is a CategoryChooserBlock for something else and this is
# why there's a category_block.py.
class CategoryChooserBlock(blocks.StructBlock):
    category = RoadrunnerCategoryChooserBlock(required=True)

    def get_context(self, value, parent_context=None):
        context = super().get_context(value, parent_context=parent_context)
        if value["category"]:
            products = (
                Product.objects.browsable()
                .filter(categories__in=[value["category"]])
                .distinct()
                .order_by("-priority", "title")
            )
            context["products"] = products
        else:
            context["products"] = Product.objects.none()
            warning = f"CategoryChooserBlock name={self.name}, definition_prefix={self.definition_prefix} is missing the category"
            page = parent_context.get("page", None)
            if page is not None:
                warning += f" on page {page} (pk={page.pk})"
        return context

    class Meta:
        template = "roadrunner_bs4/blocks/category_block.html"
        icon = "grip"
        label = "Category"
        group = "Webshop"
        preview = ["category"]



from wagtail import blocks

from oscar.core.loading import get_model

from ocyan.main.loading import get_ocyan_classes

from .viewsets import category_chooser_viewset, product_chooser_viewset

# we need a double fallback for BaseStylingBlock until we finished the
# transition to wagtail-roadrunner completely
try:
    from rr.blocks.styling import StylingBlock as BaseStylingBlock
except ImportError:
    try:
        from roadrunner.blocks.styling import BaseStylingBlock
    except ImportError:
        BaseStylingBlock = lambda: blocks.StaticBlock(
            admin_text="Install roadrunner_bs4 or roadrunner_bs5",
            template="roadrunner/stylingblockplaceholder.html",
        )

# pylint: disable=unbalanced-tuple-unpacking
AdminCategoryChooser, AdminProductChooser = get_ocyan_classes(
    "wagtail_oscar_integration.widgets", ["AdminCategoryChooser", "AdminProductChooser"]
)

Product = get_model("catalogue", "Product")


ProductChooserBlock = product_chooser_viewset.get_block_class(
    name="ProductChooserBlock",
    module_path="ocyan.plugin.wagtail_oscar_integration.blocks",
)

CategoryChooserBlock = category_chooser_viewset.get_block_class(
    name="CategoryChooserBlock",
    module_path="ocyan.plugin.wagtail_oscar_integration.blocks",
)


class ProductBlock(blocks.StructBlock):
    product = ProductChooserBlock()

    class Meta:
        icon = "arrow-right"
        label = "product"
        group = "Webshop"
        preview = ["product"]


def ProductSliderBlock(**kwargs):
    kwargs.setdefault("template", "roadrunner_bs4/blocks/product_slider.html")
    kwargs.setdefault("icon", "code")
    kwargs.setdefault("label", "products")
    kwargs.setdefault("group", "Webshop")
    return blocks.ListBlock(ProductBlock(), **kwargs)


class RoadrunnerCategoryChooserBlock(CategoryChooserBlock):
    add_choices_to_source = True


class CategoryCCHooserBlock(blocks.StructBlock):
    category = RoadrunnerCategoryChooserBlock(required=False)
    label = blocks.CharBlock(label="Alternative label", required=False)
    styling = BaseStylingBlock()

    def get_context(self, value, parent_context=None):
        context = super().get_context(value, parent_context=parent_context)
        if value["category"]:
            products = (
                Product.objects.browsable()
                .filter(categories__in=[value["category"]])
                .distinct()
                .order_by("-priority", "title")
            )
            context["products"] = products
        else:
            context["products"] = Product.objects.none()
            warning = f"CategoryChooserBlock name={self.name}, definition_prefix={self.definition_prefix} is missing the category"
            page = parent_context.get("page", None)
            if page is not None:
                warning += f" on page {page} (pk={page.pk})"
        return context

    class Meta:
        template = "roadrunner_bs4/blocks/category_c_block.html"
        icon = "link"
        label = "Category 2"
        group = "Webshop"


class RecentlyViewedBlock(blocks.StructBlock):
    styling = BaseStylingBlock()

    class Meta:
        template = "roadrunner_bs4/blocks/recently_viewed_block.html"
        icon = "view"
        label = "Recent Bekeken"
        group = "Webshop"



# -*- coding: utf-8 -*-
from ocyan.plugin.oscar.info import OSCAR_NEEDS_OXYAN

if OSCAR_NEEDS_OXYAN:
    from .entrypoint import WagtailOscar3IntegrationConfig  # noqa
else:
    from .entrypoint import WagtailOscarIntegrationConfig  # noqa

default_app_config = "ocyan.plugin.wagtail_oscar_integration.apps.OcyanPluginWagtailOscarIntegrationAppConfig"



import logging
from random import randint

from django.conf import settings
from django.core.exceptions import ObjectDoesNotExist
from django.db import connection
from django.test import override_settings
from .uwsgi_compat import cron

from ocyan.main.loading import get_ocyan_class
from ocyan.main.utils import run_exclusive

get_default_site = get_ocyan_class("wagtail_utils.utils", "get_default_site")

logger = logging.getLogger(__name__)


# pylint: disable=unused-argument
@cron(randint(0, 59), -1, -1, -1, -1, target="spooler")  # every hour at random minute
def send_customer_alerts_async(args):
    # the spooler is forked so the db connection might be closed
    try:
        if not connection.is_usable():
            connection.close()
    except AttributeError:
        connection.close_if_unusable_or_obsolete()

    try:
        if not settings.SITE_ID:
            site = get_default_site()
            with override_settings(SITE_ID=site.pk):
                run_exclusive("oscar_send_alerts")
        else:
            run_exclusive("oscar_send_alerts")
    except ObjectDoesNotExist:
        logger.error(
            "The default site can not be found, make sure the hostname in the wagtail default site matches a django site"
        )



"""
Draftail / contentstate conversion
"""

from draftjs_exporter.dom import DOM

from wagtail.admin.rich_text.converters.html_to_contentstate import LinkElementHandler

from oscar.core.loading import get_model


def product_link_entity(props):
    return DOM.create_element(
        "a",
        {
            "linktype": "product",
            "id": props.get("id"),
        },
        props["children"],
    )


def category_link_entity(props):
    return DOM.create_element(
        "a",
        {
            "linktype": "category",
            "id": props.get("id"),
        },
        props["children"],
    )


class OscarLinkElementHandler(LinkElementHandler):
    def get_model(self):
        raise NotImplementedError

    def get_attribute_data(self, attrs):
        Model = self.get_model()
        try:
            _id = int(attrs["id"])
        except (KeyError, ValueError):
            return {}

        try:
            doc = Model.objects.get(id=_id)
        except Model.DoesNotExist:
            return {"id": _id}

        return {
            "id": doc.id,
            "url": doc.get_absolute_url(),
        }


class ProductLinkElementHandler(OscarLinkElementHandler):
    def get_model(self):
        return get_model("catalogue", "Product")


class CategoryLinkElementHandler(OscarLinkElementHandler):
    def get_model(self):
        return get_model("catalogue", "Category")


ContentstateProductLinkConversionRule = {
    "from_database_format": {
        'a[linktype="product"]': ProductLinkElementHandler("PRODUCT"),
    },
    "to_database_format": {"entity_decorators": {"PRODUCT": product_link_entity}},
}


ContentstateCategoryLinkConversionRule = {
    "from_database_format": {
        'a[linktype="category"]': CategoryLinkElementHandler("CATEGORY"),
    },
    "to_database_format": {"entity_decorators": {"CATEGORY": category_link_entity}},
}



from django.template.loader import render_to_string
from django.urls import reverse


class UserBarEditProductItem:
    def __init__(self, pk):
        self.pk = pk
        self.url = reverse("dashboard:catalogue-product", kwargs={"pk": self.pk})
        self.text = "Edit this product"
        self.icon = "edit"

    def render(self, request):
        return render_to_string(
            "userbar/userbar_item.html",
            {"url": self.url, "text": self.text, "icon": self.icon},
        )


class UserBarEditCategoryItem:
    def __init__(self, pk):
        self.pk = pk
        self.url = reverse(
            "dashboard:catalogue-category-update", kwargs={"pk": self.pk}
        )
        self.text = "Edit this category"
        self.icon = "edit"

    def render(self, request):
        return render_to_string(
            "userbar/userbar_item.html",
            {"url": self.url, "text": self.text, "icon": self.icon},
        )



import lxml.etree
import lxml.html

from ocyan.main.loading import get_ocyan_classes

# pylint: disable=unbalanced-tuple-unpacking
(
    transform_buttons,
    transform_menu_items,
    transform_listings,
    transform_icons,
    transform_forms,
    transform_pagination,
) = get_ocyan_classes(
    "wagtail_oscar_integration.transform",
    [
        "transform_buttons",
        "transform_menu_items",
        "transform_listings",
        "transform_icons",
        "transform_forms",
        "transform_pagination",
    ],
)


def transform_oscar_response(response):
    tree = lxml.html.fromstring(response.content)

    transform_buttons(tree)
    transform_menu_items(tree)
    transform_listings(tree)
    transform_icons(tree)
    transform_forms(tree)
    transform_pagination(tree)

    # Back to html, add doctype first as lxml removes this for some reason.
    response.content = "<!DOCTYPE html>\n%s" % lxml.html.tostring(
        tree, encoding="unicode"
    )

    return response


def oscar_dashboard_2_wagtail(get_response):
    def middleware(request):
        response = get_response(request)

        try:
            if "dashboard" in request.resolver_match.namespaces:
                if not response.status_code == 200:
                    return response

                if not response["Content-Type"].startswith("text/html"):
                    return response

                if not request.user.is_authenticated:
                    return response

                return transform_oscar_response(response)
        except AttributeError:
            pass

        return response

    return middleware



"""Optional uWSGI cron support without making uWSGI a runtime requirement."""

try:
    from uwsgidecorators import cron  # type: ignore[import-not-found]
except ImportError:

    def cron(*_args, **_kwargs):
        """Leave the callback callable when the process is not managed by uWSGI."""

        def decorate(function):
            return function

        return decorate



from unittest import mock

from django.core.cache import cache
from django.test import TestCase, override_settings
from lxml import etree

from ocyan.main.loading import get_ocyan_class

gather_sitemaps = get_ocyan_class(
    "wagtail_oscar_integration.sitemap", "gather_sitemaps"
)


def select_urls(response):
    parsed_response = etree.fromstring(response.content)
    ns = {"sitemap": "http://www.sitemaps.org/schemas/sitemap/0.9"}
    return parsed_response.xpath("//sitemap:loc/text()", namespaces=ns)


@override_settings(
    CACHES={"default": {"BACKEND": "django.core.cache.backends.dummy.DummyCache"}}
)
class ProductsSitemapTest(TestCase):
    fixtures = [
        "wagtail_oscar_integration/catalogue",
        "wagtail_oscar_integration/sites",
    ]

    def setUp(self):
        cache.clear()
        super().setUp()

    def assertNoDuplicates(self, iterable, message=None):
        return self.assertEqual(len(set(iterable)), len(iterable), message)

    def test_no_duplicates(self):
        product_sitemap = gather_sitemaps()["products"]
        product_ids = [p.pk for p in product_sitemap().items()]
        self.assertNoDuplicates(
            product_ids, "there are no duplicate items in the product sitemap"
        )

    @mock.patch(
        "ocyan.plugin.wagtail_oscar_integration.sitemap.INCLUDE_CHILD_PRODUCTS_IN_SITEMAP",
        False,
    )
    def test_product_sitemap_browsable(self):
        response = self.client.get("/sitemap-products.xml?zult=gll")
        urls = select_urls(response)

        self.assertNoDuplicates(urls, "There should be no duplicates in the sitemap")
        self.assertListEqual(
            sorted(urls),
            [
                "http://testserver/dikke-shit-ouwe-2/",
                "http://testserver/henk-3/",
                "http://testserver/klats-6/",
                "http://testserver/second-5/",
                "http://testserver/serious-product-4/",
            ],
        )

    @mock.patch(
        "ocyan.plugin.wagtail_oscar_integration.sitemap.INCLUDE_CHILD_PRODUCTS_IN_SITEMAP",
        True,
    )
    def test_product_sitemap_include_childs(self):
        response = self.client.get("/sitemap-products.xml")
        urls = select_urls(response)

        self.assertNoDuplicates(
            urls,
            "There should be no duplicates in the sitemap, even when including children",
        )
        self.assertListEqual(
            sorted(urls),
            [
                "http://testserver/dikke-shit-ouwe-2/",
                "http://testserver/harrie-7/",
                "http://testserver/henk-3/",
                "http://testserver/klats-6/",
                "http://testserver/second-5/",
                "http://testserver/secondchild-8/",
                "http://testserver/serious-product-4/",
            ],
        )


@override_settings(
    CACHES={"default": {"BACKEND": "django.core.cache.backends.dummy.DummyCache"}}
)
class CategorySitemapTest(TestCase):
    fixtures = [
        "wagtail_oscar_integration/catalogue",
        "wagtail_oscar_integration/sites",
    ]

    def test_category_sitemap(self):
        response = self.client.get("/sitemap-categories.xml")
        urls = select_urls(response)
        self.assertListEqual(
            urls,
            [
                "http://testserver/categorie/root-1/",
                "http://testserver/categorie/grav-equip-3/",
            ],
        )


@override_settings(
    CACHES={"default": {"BACKEND": "django.core.cache.backends.dummy.DummyCache"}}
)
class PagesSitemapTest(TestCase):
    fixtures = ["wagtail_oscar_integration/pages", "wagtail_oscar_integration/sites"]

    def test_page_sitemap(self):
        response = self.client.get("/sitemap-pages.xml")

        self.assertEqual(
            response.content,
            b'<?xml version="1.0" encoding="UTF-8"?>\n<urlset xmlns="http://www.sitemaps.org/schemas/sitemap/0.9" xmlns:xhtml="http://www.w3.org/1999/xhtml">\n<url><loc>http://testserver/</loc></url>\n</urlset>\n',
        )


@override_settings(
    CACHES={"default": {"BACKEND": "django.core.cache.backends.dummy.DummyCache"}}
)
class ShopSitemapTest(TestCase):
    fixtures = ["wagtail_oscar_integration/sites"]

    def test_shop_sitemap_oxyan(self):
        response = self.client.get("/sitemap-shop.xml?oxyan")

        self.assertEqual(
            response.content,
            b'<?xml version="1.0" encoding="UTF-8"?>\n<urlset xmlns="http://www.sitemaps.org/schemas/sitemap/0.9" xmlns:xhtml="http://www.w3.org/1999/xhtml">\n<url><loc>http://testserver/offers/</loc></url>\n</urlset>\n',
        )



"""
editor-html conversion for contenteditable editors
"""

from django.utils.html import escape

from wagtail.admin.rich_text.converters import editor_html

from oscar.core.loading import get_model


class OscarModelLinkHandlerBase:
    linktype = None

    def get_model(self):
        raise NotImplementedError

    @staticmethod
    def get_db_attributes(tag):
        return {"id": tag["data-id"]}

    def expand_db_attributes(self, attrs):
        Model = self.get_model()
        try:
            prod = Model.objects.get(id=attrs["id"])
            return '<a data-linktype="%s" data-id="%d" href="%s">' % (
                self.linktype,
                prod.id,
                escape(prod.get_absolute_url()),
            )
        except Model.DoesNotExist:
            # Preserve the ID attribute for troubleshooting purposes, even though it
            # points to a missing document
            return '<a data-linktype="%s" data-id="%s">' % (self.linktype, attrs["id"])
        except KeyError:
            return '<a data-linktype="%s">' % self.linktype


class ProductLinkHandler(OscarModelLinkHandlerBase):
    linktype = "product"

    def get_model(self):
        return get_model("catalogue", "Product")


class CategoryLinkHandler(OscarModelLinkHandlerBase):
    linktype = "category"

    def get_model(self):
        return get_model("catalogue", "Category")


EditorHTMLProductLinkConversionRule = [
    editor_html.LinkTypeRule("product", ProductLinkHandler),
]


EditorHTMLCategoryLinkConversionRule = [
    editor_html.LinkTypeRule("category", CategoryLinkHandler),
]



from django.utils.translation import gettext_lazy as _

from wagtail.admin.viewsets.chooser import ChooserViewSet

from ocyan.core.fender import config
from ocyan.main.loading import get_ocyan_classes

# pylint: disable=W0632
(
    ProductChooseView,
    ProductChooseResultsView,
    CategoryChooseView,
    CategoryChooseResultsView,
) = get_ocyan_classes(
    "wagtail_oscar_integration.views",
    [
        "ProductChooseView",
        "ProductChooseResultsView",
        "CategoryChooseView",
        "CategoryChooseResultsView",
    ],
)
# pylint: disable=W0632
BaseProductChooser, BaseChooserWithImage = get_ocyan_classes(
    "wagtail_oscar_integration.chooser.widgets",
    ["BaseProductChooser", "BaseChooserWithImage"],
)

PAGE_SIZE = config.get("oscar", "dashboard_items_per_page", 21)


class ProductViewSet(ChooserViewSet):
    per_page = 13
    model = "catalogue.Product"
    icon = "shopping-cart"
    choose_view_class = ProductChooseView
    choose_results_view_class = ProductChooseResultsView
    choose_one_text = _("Choose a product")
    choose_another_text = _("Choose another product")
    base_widget_class = BaseProductChooser


class CategoryViewSet(ChooserViewSet):
    per_page = 13
    model = "catalogue.Category"
    icon = "regex"
    choose_view_class = CategoryChooseView
    choose_results_view_class = CategoryChooseResultsView
    choose_one_text = _("Choose a category")
    choose_another_text = _("Choose another category")
    base_widget_class = BaseChooserWithImage


product_chooser_viewset = ProductViewSet("product_chooser")
category_chooser_viewset = CategoryViewSet("category_chooser")



from sorl.thumbnail import default as sorl_thumbnail

from wagtail.admin.widgets.chooser import BaseChooser


class BaseChooserWithImage(BaseChooser):
    def get_image(self, instance):
        return instance.image

    def get_value_data_from_instance(self, instance):
        data = super().get_value_data_from_instance(instance)
        data.update(
            {
                "preview": self.get_primary_image_preview(instance),
            }
        )
        return data

    def get_value_data(self, value):
        if not value:
            value = None

        return super().get_value_data(value)

    def get_primary_image_preview(self, instance):
        image = self.get_image(instance)

        if image:
            thumb = sorl_thumbnail.backend.get_thumbnail(
                image,
                "165x165",
                upscale=False,
                format="JPEG",
            )
            return {
                "url": thumb.url,
                "width": thumb.width,
                "height": thumb.height,
                "alt": instance.description,
            }

        return {}


class BaseProductChooser(BaseChooserWithImage):
    def get_image(self, instance):
        image = instance.primary_image()
        return image.original if hasattr(image, "original") else image["original"]



from django import forms
from django.utils.translation import gettext_lazy as _

from wagtail.admin.forms.choosers import BaseFilterForm


class FakeSearchFilterForm(BaseFilterForm):
    q = forms.CharField(
        label=_("Search term"),
        widget=forms.TextInput(attrs={"placeholder": _("Search")}),
        required=False,
    )

    def filter(self, objects):
        objects = super().filter(objects)
        search_query = self.cleaned_data.get("q")
        if search_query:
            self.is_searching = True
            self.search_query = search_query

        return objects



from django.utils.translation import gettext as _

from wagtail import blocks

from ocyan.main.loading import get_baseclass
from ocyan.plugin.coyote.blocks.partials import CoyoteStructBlock
from ocyan.plugin.coyote.custom_blocks import IconChoiceBlock, VisualChoiceBlock
from ocyan.plugin.coyote.icons import ICONS

# pylint: disable=E0401,E0611
from ocyan.plugin.wagtail_oscar_integration.blocks import (
    CategoryChooserBlock,
    ProductChooserBlock,
)

BaseHomePageContentStreamBlock = get_baseclass(
    "coyote.stream_blocks",
    "HomePageContentStreamBlock",
    "ocyan.plugin.wagtail_oscar_integration.wagtail_oscar_integration_coyote_extension",
)


class CoyoteProductsBlock(CoyoteStructBlock):
    layout = VisualChoiceBlock(
        choices=[
            ("layout1", "coyote/layouts/products_layout_1.png"),
            ("layout2", "coyote/layouts/products_layout_2.png"),
        ]
    )
    body = blocks.RichTextBlock(required=False, label=_("Short text"))
    read_more_button = blocks.CharBlock(
        required=False, label=_("Read more button"), default=_("More articles")
    )
    read_more_link = blocks.URLBlock(required=False, label=_("Read more url"))
    products = blocks.ListBlock(
        ProductChooserBlock(), required=False, label=_("Products")
    )

    class Meta:
        template = "coyote/blocks/content/productsblock.html"


class CoyoteCategorychooser(blocks.StructBlock):
    icon = IconChoiceBlock(choices=ICONS, required=False)
    category = CategoryChooserBlock()


class CoyoteCategoryBlock(CoyoteStructBlock):
    layout = VisualChoiceBlock(
        choices=[
            ("layout1", "coyote/layouts/category_layout_1.png"),
            ("layout2", "coyote/layouts/category_layout_2.png"),
        ]
    )
    body = blocks.RichTextBlock(required=False, label=_("Short text"))
    read_more_button = blocks.CharBlock(
        required=False, label=_("Read more button"), default=_("More articles")
    )
    read_more_link = blocks.URLBlock(required=False, label=_("Read more url"))
    categories = blocks.ListBlock(
        CoyoteCategorychooser(), required=False, label=_("Categories")
    )

    class Meta:
        template = "coyote/blocks/content/categoryblock.html"


class HomePageContentStreamBlock(BaseHomePageContentStreamBlock):
    Products = CoyoteProductsBlock(icon="tag")
    Category = CoyoteCategoryBlock(icon="folder")



# -*- coding: utf-8 -*-
# pylint: disable=W0212
import re

from django.template import Context, Template
from django.urls import reverse_lazy
from django.utils.html import format_html
from django.utils.translation import gettext

import wagtail.admin.rich_text.editors.draftail.features as draftail_features
from wagtail import hooks
from wagtail.admin.menu import (
    Menu,
)
from wagtail.admin.menu import (
    MenuItem as MenuItem_,
)
from wagtail.admin.menu import (
    SubmenuMenuItem as SubmenuMenuItem_,
)
from wagtail.admin.staticfiles import versioned_static

from oscar.core.loading import get_class
from oscar.templatetags.dashboard_tags import dashboard_navigation

from ocyan.main.loading import get_ocyan_classes
from ocyan.plugin.oscar.info import OSCAR_NEEDS_OXYAN

from .rich_text import CategoryLinkHandler, ProductLinkHandler
from .rich_text.contentstate import (
    ContentstateCategoryLinkConversionRule,
    ContentstateProductLinkConversionRule,
)
from .rich_text.editor_html import (
    EditorHTMLCategoryLinkConversionRule,
    EditorHTMLProductLinkConversionRule,
)
from .viewsets import category_chooser_viewset, product_chooser_viewset

DashboardPermission = get_class("dashboard.permissions", "DashboardPermission")
# pylint: disable=unbalanced-tuple-unpacking
UserBarEditProductItem, UserBarEditCategoryItem = get_ocyan_classes(
    "wagtail_oscar_integration.userbar",
    ["UserBarEditProductItem", "UserBarEditCategoryItem"],
)


FA_REGEX = re.compile(r"^fa.? fa-(?P<icon>.*)")


class OscarMixin:
    def get_context(self, request):
        context = super().get_context(request)
        context["li_additional_classes"] = " oscar"
        return context


class SubmenuMenuItem(OscarMixin, SubmenuMenuItem_):
    pass


class MenuItem(OscarMixin, MenuItem_):
    def is_active(self, request):
        # A menuitem can lead to a different url than the parent has.
        # Wagtail will take care of activating the parent menu item.
        return request.path == str(self.url)


def from_node(node):
    # Convert an Oscar dashboard menu node to Wagtail menu item
    # Nodes with children become a submenu.
    icon = "arrow-right"
    if node.icon:
        icon_match = FA_REGEX.match(node.icon)
        if icon_match is not None:
            icon = icon_match.group("icon")

    if node.children:
        register_hook_name = f"register_oscar_item_{node.label}"
        menu = Menu(register_hook_name=register_hook_name)
        menu.initial_menu_items = [from_node(child) for child in node.children]
        return SubmenuMenuItem(label=node.label, menu=menu, icon_name=icon)
    else:
        return MenuItem(label=node.label, url=node.url, icon_name=icon)


@hooks.register("construct_main_menu")
def create_oscar_menus(request, menuitems):
    nodes = dashboard_navigation(request.user)
    menuitems.extend(from_node(node) for node in nodes)
    return menuitems


if OSCAR_NEEDS_OXYAN:

    @hooks.register("insert_global_admin_css")
    def global_admin_css():
        """Add /static/css/custom.css to the admin."""
        tpl = Template(
            """{% load compress static %}{% compress css %}<link rel="stylesheet" type="text/x-scss" href="{% static 'oscar/oxyan/dashboard.scss' %}">{% endcompress %}"""
        )
        return tpl.render(Context())

else:

    @hooks.register("insert_global_admin_css")
    def global_admin_css():
        return format_html(
            '<link rel="stylesheet" href="{}">',
            versioned_static(
                "ocyan_plugin_wagtail_oscar_integration/wagtail_oscar_dashboard.css"
            ),
        )


@hooks.register("register_admin_viewset")
def register_viewset():
    return product_chooser_viewset, category_chooser_viewset


@hooks.register("construct_wagtail_userbar")
def add_product_edit_item(request, items, page):  # pylint: disable=unused-argument
    try:
        product_slug = request.resolver_match.kwargs.get("product_slug", None)
        pk = request.resolver_match.kwargs.get("pk", None)
        if pk and product_slug:
            items.append(UserBarEditProductItem(pk))
    except AttributeError:
        pass


@hooks.register("construct_wagtail_userbar")
def add_category_edit_item(request, items, page):  # pylint: disable=unused-argument
    try:
        category_slug = request.resolver_match.kwargs.get("category_slug", None)
        pk = request.resolver_match.kwargs.get("pk", None)
        if pk and category_slug:
            items.append(UserBarEditCategoryItem(pk))
    except AttributeError:
        pass


@hooks.register("after_edit_user")
@hooks.register("after_create_user")
def configure_oscar_permissions(request, user):
    if user.is_superuser or DashboardPermission.has_dashboard_perms(user):
        user.is_staff = True
        user.save()


@hooks.register("register_rich_text_features")
def register_productlink_feature(features):
    features.register_link_type(ProductLinkHandler)

    features.register_editor_plugin(
        "draftail",
        "product-link",
        draftail_features.EntityFeature(
            {
                "type": "PRODUCT",
                "icon": "tag",
                "description": gettext("Product"),
                "chooserUrls": {
                    "productChooser": reverse_lazy("product_chooser:choose")
                },
            },
            js=["wagtailadmin/js/product-chooser-modal.js"],
        ),
    )

    features.register_converter_rule(
        "editorhtml", "product-link", EditorHTMLProductLinkConversionRule
    )
    features.register_converter_rule(
        "contentstate", "product-link", ContentstateProductLinkConversionRule
    )

    features.default_features.append("product-link")


@hooks.register("register_rich_text_features")
def register_categorylink_feature(features):
    features.register_link_type(CategoryLinkHandler)

    features.register_editor_plugin(
        "draftail",
        "category-link",
        draftail_features.EntityFeature(
            {
                "type": "CATEGORY",
                "icon": "tasks",
                "description": gettext("Category"),
                "chooserUrls": {
                    "categoryChooser": reverse_lazy("category_chooser:choose")
                },
            },
            js=["wagtailadmin/js/category-chooser-modal.js"],
        ),
    )

    features.register_converter_rule(
        "editorhtml", "category-link", EditorHTMLCategoryLinkConversionRule
    )
    features.register_converter_rule(
        "contentstate", "category-link", ContentstateCategoryLinkConversionRule
    )

    features.default_features.append("category-link")



from django.urls import path, reverse_lazy
from django.views.decorators.cache import cache_page

from ocyan.core.utils import merge_dicts
from ocyan.plugin.wagtail.block_plugin import BlockPluginConfig

from .constants import CACHE_DURATION

WOI_SETTINGS = {
    "STATICFILES_FINDERS": [
        "ocyan.plugin.wagtail_oscar_integration.utils.StaticSrcFinder"
    ],
    "WAGTAIL_FRONTEND_LOGIN_URL": reverse_lazy("customer:login"),
}


class WagtailOscarIntegrationConfig(BlockPluginConfig):
    def contribute_to_settings(self, settings):
        return merge_dicts(settings, WOI_SETTINGS)

    def get_smoketest_modules(self):
        return ["ocyan.plugin.wagtail_oscar_integration.tests.test_smoketests"]

    def contribute_to_installed_apps(self, installed_apps):
        installed_apps.django.append("django.contrib.sitemaps")
        installed_apps.ocyan.append("ocyan.plugin.wagtail_oscar_integration")
        installed_apps.wagtail.extend(
            [
                "wagtail.contrib.settings",
                "ocyan.plugin.wagtail_oscar_integration.wagtail_extension",
            ]
        )
        installed_apps.wagtail.register_extension(
            "ocyan.plugin.wagtail_oscar_integration.wagtail_oscar_integration_coyote_extension",
            "ocyan.plugin.coyote",
        )

        installed_apps.wagtail.append(
            "ocyan.plugin.wagtail_oscar_integration.wagtail_user_extension.apps.CustomUsersAppConfig",
            replaces="wagtail.users",
        )
        return installed_apps

    def contribute_to_middleware(self, middleware):
        return middleware + [
            "ocyan.plugin.wagtail_oscar_integration.middleware.oscar_dashboard_2_wagtail"
        ]

    def contribute_to_urlpatterns(self, urlpatterns):
        from django.contrib.sitemaps import views

        from ocyan.main.loading import get_ocyan_class

        gather_sitemaps = get_ocyan_class(
            "wagtail_oscar_integration.sitemap", "gather_sitemaps"
        )

        sitemaps = gather_sitemaps()

        return [
            path(
                "sitemap.xml",
                cache_page(CACHE_DURATION)(views.index),
                {"sitemaps": sitemaps, "sitemap_url_name": "sitemaps"},
            ),
            path(
                "sitemap-<section>.xml",
                cache_page(CACHE_DURATION)(views.sitemap),
                {"sitemaps": sitemaps},
                name="sitemaps",
            ),
        ] + urlpatterns

    def get_spooler_modules(self, modules):
        modules.append("ocyan.plugin.wagtail_oscar_integration.spooler")
        return modules

    def contribute_to_blocks(self, blocks):
        from .blocks import (
            CategoryCCHooserBlock,
            ProductChooserBlock,
            ProductSliderBlock,
            RecentlyViewedBlock,
        )
        from .category_block import CategoryChooserBlock

        return blocks + [
            ("Category", CategoryChooserBlock()),
            ("CategoryCBlock", CategoryCCHooserBlock(), "roadrunner"),
            ("Recent", RecentlyViewedBlock(), "roadrunner,rr"),
            ("ProductSlider", ProductSliderBlock()),
            (
                "add_to_cart",
                ProductChooserBlock(
                    template="wagtail_content_page/partials/add-to-cart.html"
                ),
                "contentpage",
            ),
        ]

    def contribute_to_maintenance(self, maintenances):
        return maintenances + [
            (
                "helpdesk_group",
                "ocyan.plugin.wagtail_oscar_integration.maintenance.helpdesk_group",
            ),
        ]


class WagtailOscar3IntegrationConfig(WagtailOscarIntegrationConfig):
    def contribute_to_middleware(self, middleware):
        return middleware

    def contribute_to_installed_apps(self, installed_apps):
        installed_apps = super().contribute_to_installed_apps(installed_apps)
        installed_apps.ocyan.register_extension(
            "ocyan.plugin.wagtail_oscar_integration.oxyan_wagtail_oscar_integration",
            "ocyan.plugin.wagtail_oscar_integration",
        )
        return installed_apps



from ocyan.core.fender import config

INCLUDE_CHILD_PRODUCTS_IN_SITEMAP = config.get(
    "wagtail_oscar", "sitemap_include_child_products", False
)

CACHE_DURATION = config.get("wagtail_oscar", "sitemap_cache_duration", 86400)

PAGE_SIZE = config.get("wagtail_oscar", "sitemap_page_size", 50000)
ALTERNATES = config.get("wagtail_oscar", "sitemap_alternates", True)



from wagtail.users.views.users import UserViewSet as WagtailUserViewSet

from .forms import WagtailUserCreateForm, WagtailUserEditForm


class UserViewSet(WagtailUserViewSet):
    def get_form_class(self, for_update=False):
        """
        If for_update=True, this is editing an existing user,
        else it's for creating a new user.
        """
        if for_update:
            return WagtailUserEditForm
        return WagtailUserCreateForm



from wagtail.users.apps import WagtailUsersAppConfig


class CustomUsersAppConfig(WagtailUsersAppConfig):
    user_viewset = "ocyan.plugin.wagtail_oscar_integration.wagtail_user_extension.viewsets.UserViewSet"



# -*- coding: utf-8 -*-
from django.apps import AppConfig


class OcyanPluginWagtailOscarIntegrationAppConfig(AppConfig):
    name = "ocyan.plugin.wagtail_oscar_integration"
    label = "wagtail_oscar_integration"



import lxml.etree


def has_class(klass):
    return "[contains(concat(' ',normalize-space(@class),' '),' " + klass + " ')]"


def add_class(elem, klasses):
    if not isinstance(klasses, list):
        klasses = [klasses]
    existing = elem.attrib.get("class", "").split()
    to_add = [klass for klass in klasses if klass not in existing]
    if to_add:
        elem.attrib["class"] = " ".join(existing + to_add)


def remove_class(elem, klasses):
    if not isinstance(klasses, list):
        klasses = [klasses]

    elem.attrib["class"] = " ".join(
        [x for x in elem.attrib.get("class", "").split() if x not in klasses]
    )


def get_icon(element, clear=False):
    icon = None
    for sub in element.xpath('*[contains(@class, "icon")]'):
        for klass in sub.attrib.get("class", "").split(" "):
            if klass.startswith("icon-"):
                icon = klass
                break

        if icon and clear:
            remove_class(sub, [icon, "icon"])
    return icon


def transform_buttons(tree):
    # btn -> button
    for elm in tree.find_class("btn"):
        primary = "btn-primary" in elm.attrib["class"]
        remove_class(elm, ["btn-primary", "btn-lg", "btn"])
        add_class(elm, "button")

        # icon
        icon = get_icon(elm, clear=True)
        if icon:
            to_add = ["icon", icon]
            if primary:
                to_add += ["bicolor"]
            add_class(elm, to_add)


def transform_menu_items(tree):
    # grab active main menuitem and insert its icon in the header
    for menuitem in tree.xpath("//nav//li" + has_class("menu-active")):
        icon = get_icon(menuitem)
        if icon:
            for h1 in tree.xpath("//header//h1"):
                add_class(h1, ["icon", icon])


def transform_listings(tree):
    # listings
    to_remove = "table table-striped table-bordered table-hover".split()
    for table in tree.xpath("//table" + has_class("table")):
        remove_class(table, to_remove)
        table.attrib["class"] = "listing " + table.attrib["class"]

        for caption in table.xpath("caption"):
            add_class(caption, "table-headers")

        for button in table.xpath("caption//a" + has_class("button")):
            remove_class(button, "button")


def transform_icons(tree):
    # icons
    for elem in tree.xpath("//*[contains(@class, 'icon-')]"):
        add_class(elem, "icon")


def transform_forms(tree):
    # forms
    for form in tree.xpath("//form"):
        # Disable all html5 validation in the backend.
        form.attrib["novalidate"] = ""

        remove_class(form, "well")

        for elm in form.xpath("*" + has_class("form-group")):
            remove_class(elm, "form-group")
            elm.tag = "li"  # this should be an li
            if elm.xpath("label" + has_class("required")):
                add_class(elm, "required")

            for div in elm.iterchildren("div"):
                div.attrib["class"] = "field-content"

                for form_control in div.xpath("*" + has_class("form-control")):
                    remove_class(form_control, "form-control")

            # wrap in a div[class='field']
            field_div = lxml.etree.Element("div")
            field_div.attrib["class"] = "field"
            for child in reversed(elm.getchildren()):
                child.addprevious(field_div)
                field_div.insert(0, child)

        # remove the "*" marking required fields. Wagtail uses css for that.
        for span in form.xpath('//span[text()="*"]'):
            span.getparent().remove(span)

        # Now wrap all the li's of the form in a ul
        ul = lxml.etree.Element("ul")
        ul.attrib["class"] = "fields"
        for li in reversed(form.xpath("li")):
            li.addprevious(ul)
            ul.insert(0, li)


def transform_pagination(tree):
    # Pagination
    for elem in tree.xpath("//ul" + has_class("pager")):
        replacement = lxml.etree.Element("div")
        replacement.attrib["class"] = "pagination col12"

        # current page text
        current = elem.xpath("//li" + has_class("current"))[0].text.strip()
        p_el = lxml.etree.Element("p", text=current)
        p_el.text = current
        replacement.append(p_el)

        # prev/next
        prev_el = elem.xpath("//li" + has_class("previous"))
        next_el = elem.xpath("//li" + has_class("next"))
        if prev_el or next_el:
            wrapper = lxml.etree.Element("ul")
            if prev_el:
                prev_el = prev_el[0]
                prev_el.attrib["class"] = "prev"
                prev_a = prev_el.xpath("//a")[0]
                prev_a.attrib["class"] = "icon icon-arrow-left"
                wrapper.append(prev_el)
            if next_el:
                next_el = next_el[0]
                next_a = next_el.xpath("//a")[0]
                next_a.attrib["class"] = "icon icon-arrow-right"
                wrapper.append(next_el)
            replacement.append(wrapper)
        elem.getparent().replace(elem, replacement)



import unittest

from bs4 import BeautifulSoup
from django.test import TestCase

from ocyan.core.fender import config

from ocyan.plugin.wagtail_oscar_integration.constants import (
    INCLUDE_CHILD_PRODUCTS_IN_SITEMAP,
)

LANGUAGE_CODE = config.get("django", "language_code", "en")


class SiteMapSmokeTest(TestCase):
    fixtures = [
        "wagtail_oscar_integration/catalogue",
        "wagtail_oscar_integration/sites",
        "wagtail_oscar_integration/pages",
    ]

    @unittest.skipIf(INCLUDE_CHILD_PRODUCTS_IN_SITEMAP, "Children in sitemap")
    def test_product_sitemap_browsable(self):
        response = self.client.get("/sitemap-products.xml?zult-hdk")
        soup = BeautifulSoup(response.content.decode())

        num_expected_urls = 5
        if config.i18n_enabled:
            num_expected_urls = num_expected_urls * len(
                config.get("i18n", "languages", [LANGUAGE_CODE])
            )

        self.assertEqual(len(soup.find_all("url")), num_expected_urls)
        urls = soup.find_all("loc")
        self.assertEqual(
            len(urls), len(set(urls)), "There should be no duplicates in the sitemap"
        )

    @unittest.skipUnless(INCLUDE_CHILD_PRODUCTS_IN_SITEMAP, "No children in sitemap")
    def test_product_sitemap_browsable_children(self):
        response = self.client.get("/sitemap-products.xml?lalalalal")
        soup = BeautifulSoup(response.content.decode())

        num_expected_urls = 7
        if config.i18n_enabled:
            num_expected_urls = num_expected_urls * len(
                config.get("i18n", "languages", [LANGUAGE_CODE])
            )

        self.assertEqual(len(soup.find_all("url")), num_expected_urls)
        urls = soup.find_all("loc")
        self.assertEqual(
            len(urls), len(set(urls)), "There should be no duplicates in the sitemap"
        )

    def test_category_sitemap(self):
        response = self.client.get("/sitemap-categories.xml")
        soup = BeautifulSoup(response.content.decode())

        num_expected_urls = 2
        if config.i18n_enabled:
            num_expected_urls = num_expected_urls * len(
                config.get("i18n", "languages", [LANGUAGE_CODE])
            )

        self.assertEqual(len(soup.find_all("url")), num_expected_urls)
        urls = soup.find_all("loc")
        self.assertEqual(
            len(urls), len(set(urls)), "There should be no duplicates in the sitemap"
        )

    def test_page_sitemap(self):
        response = self.client.get("/sitemap-pages.xml")
        soup = BeautifulSoup(response.content.decode())

        self.assertEqual(len(soup.find_all("url")), 1)
        urls = soup.find_all("loc")
        self.assertEqual(
            len(urls), len(set(urls)), "There should be no duplicates in the sitemap"
        )



from django import forms
from django.utils.translation import gettext_lazy as _

from wagtail.users.forms import UserCreationForm, UserEditForm


class WagtailUserEditForm(UserEditForm):
    """
    Override the wagtail userform so we can edit users created by oscar without
    being required to enter a first and last name.
    """

    first_name = forms.CharField(required=False, label=_("First Name"))
    last_name = forms.CharField(required=False, label=_("Last Name"))
    is_superuser = forms.BooleanField(
        label=_("Administrator"),
        required=False,
        help_text=_(
            "Administrators have full access to manage any object or setting. (implies Webshop administrator)"
        ),
    )
    is_staff = forms.BooleanField(
        label=_("Webshop administrator"),
        required=False,
        help_text=_(
            "Gives full access to any webshop items, like product management and orders."
        ),
    )

    class Meta(UserEditForm.Meta):
        fields = UserEditForm.Meta.fields | {"is_staff"}


class WagtailUserCreateForm(UserCreationForm):
    """
    Override the wagtail userform so we can edit users created by oscar without
    being required to enter a first and last name.
    """

    first_name = forms.CharField(required=False, label=_("First Name"))
    last_name = forms.CharField(required=False, label=_("Last Name"))
    is_superuser = forms.BooleanField(
        label=_("Administrator"),
        required=False,
        help_text=_(
            "Administrators have full access to manage any object or setting. (implies Webshop administrator)"
        ),
    )
    is_staff = forms.BooleanField(
        label=_("Webshop administrator"),
        required=False,
        help_text=_(
            "Gives full access to any webshop items, like product management and orders."
        ),
    )

    class Meta(UserCreationForm.Meta):
        fields = UserCreationForm.Meta.fields | {"is_staff"}



from django.core.paginator import InvalidPage
from django.http import Http404

from wagtail.admin.views.generic.chooser import (
    BaseChooseView,
)

from .forms import FakeSearchFilterForm


# pylint: disable=abstract-method
class BaseSearchChooseView(BaseChooseView):
    ordering = True
    filter_form_class = FakeSearchFilterForm
    create_action_label = None
    elasticsearch_api_class = None

    def can_create(self):
        return False

    def apply_object_list_ordering(self, objects):
        return objects

    def consult_search_engine(self, request):
        self.filter_form.filter(None)

        page = int(request.GET.get("p", 1)) - 1
        from_ = self.per_page * page
        # pylint: disable=E1102
        result = (
            self.elasticsearch_api_class()
            .paginated_search(
                from_=from_,
                to=self.per_page,
                query_string=self.filter_form.search_query,
                search_type="cross_fields",
                search_operator="and",
                filters=[],
            )
            .page(page + 1)
        )
        # pylint: disable=attribute-defined-outside-init
        self.paginator = result.paginator

        return result

    def get_results_page(self, request):
        objects = self.get_object_list()
        objects = self.filter_object_list(objects)

        if self.filter_form.is_searching:
            return self.consult_search_engine(request)

        # pylint: disable=attribute-defined-outside-init
        self.paginator = self.paginator_class(objects, per_page=self.per_page)
        try:
            return self.paginator.page(request.GET.get("p", 1))
        except InvalidPage:
            raise Http404



from wagtail.contrib.settings.templatetags.wagtailsettings_tags import (
    SettingsNode,
    register,
)

from ocyan.main.loading import get_ocyan_class

get_wagtailsettings = get_ocyan_class("wagtail_utils.utils", "get_wagtailsettings")


@register.simple_tag(takes_context=True)
def get_settings(context, use_default_site=False):
    try:
        context["settings"] = SettingsNode.get_settings_object(
            context, use_default_site
        )
    except RuntimeError:
        if not "site" in context:
            raise
        context["settings"] = get_wagtailsettings(context["site"].domain)
    return ""



# -*- coding: utf-8 -*-
from setuptools import find_namespace_packages, setup

install_requires: list = [
    "ocyan.core",
    "ocyan.plugin.wagtail",
    "ocyan.plugin.wagtail_utils",
    "ocyan.plugin.oscar",
    "ocyan.plugin.oscar_elasticsearch",
    "lxml",
]

extras_require: dict = {
    "test": [
        "ocyan.plugin.testing",
        "wheel",
        "empty_testproject",
        "pylint-django",
        "isort",
        "ruff",
        "vdt.versionplugin.wheel",
        "coverage",
        "beautifulsoup4",
        "ocyan.plugin.coyote",
    ]
}

PACKAGE_CLASSIFIERS = [
    "License :: Other/Proprietary License",
    "Framework :: Ocyan",
    "Environment :: Plugins",
]

setup(
    name="ocyan.plugin.wagtail_oscar_integration",
    version="2.0.2rc1",
    url="https://git.mandelblog.com/mandel-plugins/ocyan.plugin.wagtail_oscar_integration",
    author="Mandel",
    author_email="info@mandelblog.com",
    description="Integrates Oscar dashboard with Wagtail dashboard.",
    packages=find_namespace_packages(include=["ocyan.*"]),
    include_package_data=True,
    python_requires=">=3.6",
    install_requires=install_requires,
    extras_require=extras_require,
    classifiers=PACKAGE_CLASSIFIERS,
)



from django.contrib.auth.models import Group
from django.core.management import call_command


def helpdesk_group():
    helpdesk_groups = Group.objects.filter(
        permissions__codename="dashboard_access",
        permissions__content_type__app_label="partner",
        permissions__content_type__model="partner",
    )
    if not helpdesk_groups.exists():
        call_command("loaddata", "wagtail_oscar_integration/helpdesk")
        return "helpdesk group created"

    return "A helpdesk group allready exists %s" % helpdesk_groups



import math

from django.conf import settings
from django.contrib.sitemaps import Sitemap
from django.db.models import Exists, OuterRef, Q
from django.urls import reverse

from wagtail.contrib.sitemaps import sitemap_generator
from wagtail.models import Locale, Page

from oscar.core.loading import get_model

from ocyan.core.fender import config
from ocyan.main.loading import get_ocyan_class

from .constants import ALTERNATES, INCLUDE_CHILD_PRODUCTS_IN_SITEMAP, PAGE_SIZE

get_offline_request = get_ocyan_class(
    "wagtail_utils.utils",
    "get_offline_request",
)

Product = get_model("catalogue", "Product")
ProductCategory = get_model("catalogue", "ProductCategory")
Category = get_model("catalogue", "Category")


class FastI18nSitemap(Sitemap):
    i18n = config.i18n_enabled
    alternates = config.i18n_enabled and ALTERNATES
    limit = PAGE_SIZE

    def __init__(self):
        super().__init__()
        if self.i18n:
            # divide the limit per page by the number of languages.
            # we are going to print each url in each language.
            _num_languages = len(self._languages())
            self.limit = math.floor(self.__class__.limit / _num_languages)

    def _items(self):
        return self.items()

    def _urls(self, page, protocol, domain):
        if not self.i18n:
            return super()._urls(page, protocol, domain)
        else:
            urls = []
            latest_lastmod = None
            all_items_lastmod = True  # track if all items have a lastmod

            paginator_page = self.paginator.page(page)
            for _item in paginator_page.object_list:
                for lang in self._languages():
                    # print each url in each language
                    item = (_item, lang)
                    loc = f"{protocol}://{domain}{self._location(item)}"
                    priority = self._get("priority", item)
                    lastmod = self._get("lastmod", item)

                    if all_items_lastmod:
                        all_items_lastmod = lastmod is not None
                        if all_items_lastmod and (
                            latest_lastmod is None or lastmod > latest_lastmod
                        ):
                            latest_lastmod = lastmod

                    url_info = {
                        "item": item,
                        "location": loc,
                        "lastmod": lastmod,
                        "changefreq": self._get("changefreq", item),
                        "priority": str(priority if priority is not None else ""),
                        "alternates": [],
                    }

                    if self.alternates:
                        for lang_code in self._languages():
                            loc = f"{protocol}://{domain}{self._location(item, lang_code)}"
                            url_info["alternates"].append(
                                {
                                    "location": loc,
                                    "lang_code": lang_code,
                                }
                            )
                        if self.x_default:
                            lang_code = settings.LANGUAGE_CODE
                            loc = f"{protocol}://{domain}{self._location(item, lang_code)}"
                            loc = loc.replace(f"/{lang_code}/", "/", 1)
                            url_info["alternates"].append(
                                {
                                    "location": loc,
                                    "lang_code": "x-default",
                                }
                            )

                    urls.append(url_info)

            if all_items_lastmod and latest_lastmod:
                # pylint: disable=attribute-defined-outside-init
                self.latest_lastmod = latest_lastmod

            return urls


class WagtailSitemap(sitemap_generator.Sitemap):
    def items(self):
        # Start with an empty QuerySet
        combined_queryset = Page.objects.none()

        # Iterate through all locales
        for locale in Locale.objects.all():
            translated_root_page = (
                self.get_wagtail_site().root_page.get_translation_or_none(locale)
            )

            if translated_root_page is not None:
                locale_pages = (
                    translated_root_page.get_descendants(inclusive=True)
                    .live()
                    .public()
                    .defer_streamfields()
                    .specific()
                    .order_by("path")
                )
                combined_queryset = combined_queryset.union(locale_pages)

        return combined_queryset

    def _urls(self, page, protocol, domain):
        self.request = get_offline_request(domain)
        return super()._urls(page, protocol, domain)


class ShopSitemap(FastI18nSitemap):
    def items(self):
        return ["offer:list"]

    # pylint: disable=arguments-renamed
    def location(self, item):
        return reverse(item)


class ProductSitemap(FastI18nSitemap):
    def lastmod(self, item):
        return item.date_updated

    def items(self):
        objects = Product._default_manager  # pylint: disable=protected-access
        if INCLUDE_CHILD_PRODUCTS_IN_SITEMAP:
            return objects.filter(
                Q(
                    Exists(ProductCategory.objects.filter(product_id=OuterRef("pk"))),
                    structure__in=[Product.PARENT, Product.STANDALONE],
                )
                | Q(
                    Exists(
                        ProductCategory.objects.filter(product_id=OuterRef("parent_id"))
                    ),
                    structure=Product.CHILD,
                ),
                is_public=True,
            ).only("id", "slug", "date_updated")
        else:
            return (
                objects.filter(
                    is_public=True,
                    structure__in=[Product.PARENT, Product.STANDALONE],
                )
                .exclude(productcategory__isnull=True)
                .only("id", "slug", "date_updated")
            )


class CategorySitemap(FastI18nSitemap):
    def items(self):
        return Category.objects.filter(is_public=True).only("id", "slug", "code")


def gather_sitemaps():
    return {
        "pages": WagtailSitemap,
        "shop": ShopSitemap,
        "products": ProductSitemap,
        "categories": CategorySitemap,
    }



from unittest import TestCase

from ocyan.plugin.wagtail_oscar_integration.uwsgi_compat import cron


class CronCompatibilityTests(TestCase):
    def test_fallback_keeps_cron_callback_callable(self):
        @cron(0, -1, -1, -1, -1, target="spooler")
        def task(arguments):
            return arguments

        self.assertEqual(task({"site_id": 1}), {"site_id": 1})



from django.utils.translation import gettext_lazy as _

from wagtail.admin.ui.tables import BooleanColumn, DateColumn, TitleColumn
from wagtail.admin.views.generic.chooser import (
    ChooseResultsViewMixin,
    ChooseViewMixin,
)

from oscar.core.loading import get_class

from .chooser.views import BaseSearchChooseView
from .tables import ImageTitleColumn

ProductElasticsearchIndex = get_class("search.api.product", "ProductElasticsearchIndex")
CategoryElasticsearchIndex = get_class(
    "search.api.category", "CategoryElasticsearchIndex"
)


# pylint: disable=abstract-method
class BaseCategoryChooseView(BaseSearchChooseView):
    elasticsearch_api_class = CategoryElasticsearchIndex

    @property
    def columns(self):
        return [
            ImageTitleColumn(
                name="image",
                label=_("Image"),
                url_name=self.chosen_url_name,
                link_attrs={"data-chooser-modal-choice": True},
            ),
            TitleColumn(
                name="full_name",
                label=_("Category"),
                url_name=self.chosen_url_name,
                link_attrs={"data-chooser-modal-choice": True},
            ),
            TitleColumn(
                name="get_num_children",
                label=_("Children"),
                url_name=self.chosen_url_name,
                link_attrs={"data-chooser-modal-choice": True},
            ),
            BooleanColumn(name="is_public", label=_("Public")),
        ]


class CategoryChooseView(ChooseViewMixin, BaseCategoryChooseView):
    pass


class CategoryChooseResultsView(ChooseResultsViewMixin, BaseCategoryChooseView):
    pass


def image_accessor(product):
    image = product.primary_image()
    try:
        return image.original
    except AttributeError:
        return image.get("original")


class BaseProductChooseView(BaseSearchChooseView):
    elasticsearch_api_class = ProductElasticsearchIndex

    @property
    def columns(self):
        return [
            ImageTitleColumn(
                name="primary_image",
                label=_("Image"),
                url_name=self.chosen_url_name,
                accessor=image_accessor,
                link_attrs={"data-chooser-modal-choice": True},
            ),
            TitleColumn(
                name="upc",
                label=_("UPC"),
                url_name=self.chosen_url_name,
                link_attrs={"data-chooser-modal-choice": True},
            ),
            TitleColumn(
                name="structure",
                label=_("Structure"),
                url_name=self.chosen_url_name,
                link_attrs={"data-chooser-modal-choice": True},
            ),
            TitleColumn(
                name="title",
                label=_("Title"),
                url_name=self.chosen_url_name,
                link_attrs={"data-chooser-modal-choice": True},
            ),
            DateColumn(name="date_updated", label=_("Modified")),
            BooleanColumn(name="is_public", label=_("Public")),
        ]


class ProductChooseView(ChooseViewMixin, BaseProductChooseView):
    pass


class ProductChooseResultsView(ChooseResultsViewMixin, BaseProductChooseView):
    pass
